<?php

/**
 * Automated Recurring Billing (ARB) API includes the following functions:
 * 1. ARBCreateSubscriptionRequest
 * 2. ARBUpdateSubscriptionRequest
 * 3. ARBGetSubscriptionStatusRequest
 * 4. ARBCancelSusbscriptionRequest
 * Each API submission may contain only one ARB request. 
 * -
 * Response and Setup
 * ------------------
 * Once each scheduled transaction in a subscription has been submitted, which is usually at 2 AM PST for ARB
 * transactions, the merchant will receive an email from the payment gateway indicating the transaction status.
 * The merchant can also configure their account in the Merchant Interface to receive the following ARB emails:
 * 1. Daily Transaction Summary.
 * 2. Failed Transaction Notice.
 * 3. Subscription Due for Expiration.
 * 4. Credit Card Expiration.
 * 5. Subscription Suspension.
 * 6. Subscription Termination.
 * 7. Subscription Expiration.
 *
 * To select which ARB emails to receive:
 * 1. Log on to the Merchant Interface at https://secure.authorize.net
 * 2. Click User Administration under Account in the main menu on the left
 * 3. Select the user you would like to edit and click Edit User
 * 4. Click Edit Profile Information under Profile and Security Settings
 * 5. Under the Automated Recurring Billing (ARB) Emails section, click to select or deselect which emails
 *    the user should receive
 * 6. Click Submit to save the changes
 * -
 * Minimum Requirement
 * -------------------
 * Following are the minimum requirements.
 * 1. Merchant must have a merchant bank account that allows Internet transactions.
 * 2. Merchant must have an active Authorize.Net Card Not Present Payment Gateway account
 * 3. Merchant must be signed up for the Authorize.Net ARB service.
 * 4. Test Mode must be disabled.
 * 5. Merchant must store account authentication data securely (for example, API login ID, transaction key).
 * -
 * Referneces
 * ----------
 * Guide: http://www.authorize.net/support/ARB_guide.pdf
 * XML Schema: https://api.authorize.net/xml/v1/schema/AnetApiSchema.xsd
 * -
 * This extension is to submit Automated Recurring Billing(ARB), or subscription-based, payments to
 * the Authorize.Net Payment Gateway directly from a Web site using extensible markup language (XML).
 * -
 * ARB subscriptions do not process transactions in real-time.
 * -
 * NOTE:
 * A successful creation of an ARB subscription transaction is not an indicator that the subscription payments
 * that process through your account will be successful. ARB subscription transactions process at approximate-
 * ly 2am PST on their scheduled payment dates. This means that the  first scheduled transaction  will  not be
 * sent to the customers bank for authorization until approximately 2am PST on the start date that you specify
 * upon creating the subscription in your account.
 *
 * If you create a subscription with a start date that equals the  creation date,  the first scheduled payment
 * will not process until after 2am the following day.
 * 
 * If you wish to validate your customers payment information before creating their subscription in your
 * account, please use one of the real-time transaction processing methods, such as the
 * AIM(Advanced Integration Method).
 * -
 * @author Deepak Pradhan
 * @version 0.01
 * @package AuthorizeNet.Base
 * @subpackage API.ARB
 */
Yii::import('application.extensions.authNet.authBase');

class authNetARB extends authBase {

	/**
	 * @var string API operation={create,update,cancel,delete}
	 */
	public $operation;
	/**
	 * @var array intermediate input - a multi-dimensional array depended to operation type
	 */
	public $params = array();
	/**
	 * @var mixed XML input to API made from $params
	 */
	protected $xml;
	/**
	 * @var mixed XML output from API
	 */
	protected $response;
	/**
	 * @var boolean true if transaction is success
	 */
	public $ref_id;
	public $success = false;
	public $result_code;
	public $code;
	public $text;
	public $subscription_id;
	public $status;
	public $next_action;
	public $message;


	/**
	 * Invokes API call
	 */
	public function run() {
		switch ($this->operation) {
			case 'create':
			case 'update':
			case 'cancel':
			case 'status':
				$this->setApi('ARB');
				$this->setXML($this->operation);
				$this->callAPI_curl('create');
				// check reponse and result in the calling controller
				break;
			default:
				// todo: trap error
				break;
		}

	}


	/**
	 * Convert the multi-dimensional array to XML for ARB call
	 * @uses array2XML
	 */
	protected function setXML($operation) {
		switch ($operation) {
			case 'create':
				$xml = "<ARBCreateSubscriptionRequest></ARBCreateSubscriptionRequest>";
				break;
			case 'update':
				$xml = "<ARBUpdateSubscriptionRequest></ARBUpdateSubscriptionRequest>";
				break;
			case 'status':
				$xml = "<ARBGetSubscriptionStatusRequest></ARBGetSubscriptionStatusRequest>";
				break;
			case 'cancel':
				$xml = "<ARBCancelSusbscriptionRequest></ARBCancelSusbscriptionRequest>";
				break;
		}
		$xml = new SimpleXMLElement($xml);
		$xml->addAttribute('xmlns', 'AnetApi/xml/v1/schema/AnetApiSchema.xsd');
		$this->xml = $this->array2XML($xml, $this->params);

	}


	/**
	 * Takes the pre-structured multi-dimensional array and makes the XML
	 * @uses array2XML_helper
	 * @param <type> $xml
	 * @param <type> $array
	 */
	private function array2XML($xml, $array) {
		return $this->array2XML_helper($array, $xml)->asXML();

	}


	/**
	 * Loop and add the Child, omits any param that is not assigned a value
	 * The params must be validated successfully in authFormARB
	 * @param array $arr
	 * @param SimpleXMLElement $xml_t
	 * @return SimpleXMLElement
	 */
	private function array2XML_helper(array $arr, SimpleXMLElement $xml_t) {
		foreach ($arr as $k=>$v) {
			if (is_array($v))
				$this->array2XML_helper($v, $xml_t->addChild($k));
			elseif (isset($v))
				$xml_t->addChild($k, $v);
		}
		return $xml_t;

	}


	/**
	 * function to send XML request to ARB API
	 * This function is direct copy php_arb_xml/authnetfunction.php
	 * There is room to make it more robust
	 * @param <type> $api_url
	 * @param <type> $content
	 * @return <type>
	 */
	protected function callAPI_curl() {
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $this->api_url);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_HTTPHEADER, Array("Content-Type: text/xml"));
		curl_setopt($ch, CURLOPT_HEADER, 1);
		curl_setopt($ch, CURLOPT_POSTFIELDS, $this->xml);
		curl_setopt($ch, CURLOPT_POST, 1);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, 0);

		$this->response = curl_exec($ch);

		if ($this->response)
			$this->parse_response();
		else {
			$this->approved = false;
			$this->declined = false;
			$this->success = false;
			$this->status = 'custom';
			$this->code = 'C99999';
			$this->text = 'ERROR: API has NO response!';
		}
		curl_close($ch);

	}


	/**
	 * function to parse ARB's XML response
	 * This function is direct copy php_arb_xml/authnetfunction.php
	 * Only modification is the name/value pair for the returned array
	 * @uses substring_between
	 * @param mixed $content XML response form API call
	 * @return array
	 */
	protected function parse_response() {
		$this->ref_id = $this->substring_between('<refId>', '</refId>');
		$this->result_code = $this->substring_between('<resultCode>', '</resultCode>');
		$this->code = $this->substring_between('<code>', '</code>');
		$this->text = $this->substring_between('<text>', '</text>');
		$this->subscription_id = $this->substring_between('<subscriptionId>', '</subscriptionId>');
		$this->success = ($this->result_code=='Ok') ? true : false;
		$this->next_action = $this->determineNextAction();

		$path = Yii::app()->getExtensionPath();
		include_once($path.'/authNet/codeDefs/arb_codes.php');

		$this->message = isset($err[$code]) ? $err[$code] : null;

		if ($this->operation=='status') {
			// possible vals = active,expired,suspended,canceled,terminated
			$this->status = $this->substring_between('<status>', '</status>');
		}

	}


	/**
	 * Alternate routine to substring_between - not tested
	 * @param <type> $start
	 * @param <type> $end
	 * @return <type>
	 */
	private function parseXML($start, $end) {
		return preg_replace('|^.*?'.$start.'(.*?)'.$end.'.*?$|i',
			'$1', $this->response);

	}


	/**
	 * This list is not complete. Read the manual and make changes here
	 * @param <type> $code
	 */
	private function determineNextAction() {
		switch ($this->code) {

			case 'I00001':
				return 'process_order';
				break;

			case 'C99999':
				return 'no_response';
				break;

			case 'E00013': // The field is invalid.
			case 'E00014': // A required field is not present
			case 'E00015': // The field length is invalid.
			case 'E00016': // The field type is invalid.
			case 'E00017': // The startDate cannot occur in the past.
			case 'E00018': // The credit card expires before the subscription startDate
			case 'E00029': // Payment information is required.
				return 're_validate';
				break;

			default: // Other errors between E00001 thru E00045
				$this->error = true;
				return 'error_exit';
				break;
		}

	}


	/**
	 * helper function for parsing ARB response
	 * This function is direct copy php_arb_xml/authnetfunction.php
	 * @param <type> $start
	 * @param <type> $end
	 * @return <type>
	 */
	private function substring_between($start, $end) {
		$haystack = $this->response;
		if (strpos($haystack, $start)===false||strpos($haystack, $end)===false) {
			return false;
		} else {
			$start_position = strpos($haystack, $start)+strlen($start);
			$end_position = strpos($haystack, $end);
			return substr($haystack, $start_position, $end_position-$start_position);
		}

	}


}